export function tintColor(color, tint) {
  if (!color) {
    return color
  }

  const channel = getColorChannels(color)
  let red = channel.red
  let green = channel.green
  let blue = channel.blue
  const alpha = channel.alpha

  if (tint === 0) {
    return [red, green, blue].join(',')
  } else {
    red += Math.round(tint * (255 - red))
    green += Math.round(tint * (255 - green))
    blue += Math.round(tint * (255 - blue))
    red = red.toString(16)
    green = green.toString(16)
    blue = blue.toString(16)

    return _.isNil(alpha) ? `#${red}${green}${blue}` : `#${red}${green}${blue}${alpha.toString(16)}`
  }
}

export function shadeColor(color, shade) {
  if (!color) {
    return color
  }

  const channel = getColorChannels(color)
  let red = channel.red
  let green = channel.green
  let blue = channel.blue
  const alpha = channel.alpha

  red = Math.round((1 - shade) * red)
  green = Math.round((1 - shade) * green)
  blue = Math.round((1 - shade) * blue)
  red = red.toString(16)
  green = green.toString(16)
  blue = blue.toString(16)

  return _.isNil(alpha) ? `#${red}${green}${blue}` : `#${red}${green}${blue}${alpha.toString(16)}`
}

export function mixColor(color, percent) {
  if (!color) {
    return color
  }

  let { red, green, blue, alpha } = getColorChannels(color)
  if (percent > 0) {
    // shade given color
    red *= 1 - percent
    green *= 1 - percent
    blue *= 1 - percent
  } else {
    // tint given color
    red += (255 - red) * percent
    green += (255 - green) * percent
    blue += (255 - blue) * percent
  }
  return _.isNil(alpha)
    ? `rgb(${Math.round(red)}, ${Math.round(green)}, ${Math.round(blue)})`
    : `rgb(${Math.round(red)}, ${Math.round(green)}, ${Math.round(blue)}, ${Math.round(alpha)})`
}

function getColorChannels(color) {
  color = color.replace('#', '')
  if (/^[0-9a-fA-F]{3}$/.test(color)) {
    color = color.split('')
    for (let i = 2; i >= 0; i--) {
      color.splice(i, 0, color[i])
    }
    color = color.join('')
  } else if (/^[0-9a-fA-F]{4}$/.test(color)) {
    color = color.split('')
    for (let i = 3; i >= 0; i--) {
      color.splice(i, 0, color[i])
    }
    color = color.join('')
  }

  if (/^[0-9a-fA-F]{6}$/.test(color)) {
    return {
      red: parseInt(color.slice(0, 2), 16),
      green: parseInt(color.slice(2, 4), 16),
      blue: parseInt(color.slice(4, 6), 16),
    }
  } else if (/^[0-9a-fA-F]{8}$/.test(color)) {
    return {
      red: parseInt(color.slice(0, 2), 16),
      green: parseInt(color.slice(2, 4), 16),
      blue: parseInt(color.slice(4, 6), 16),
      alpha: parseInt(color.slice(6, 8), 16),
    }
  }

  return {
    red: 255,
    green: 255,
    blue: 255,
  }
}
